Package: spada
Title: A 'shiny' Package for Data Analysis
Version: 0.1.6
Authors@R: 
    person("Luís Gustavo", "Schuck", email="luisgustavoschuck@yahoo.com.br",  
    role = c("aut", "cre"))
Maintainer: Luís Gustavo Schuck <luisgustavoschuck@yahoo.com.br>
Description: Provides a 'shiny' application with a user-friendly interface 
  for interactive data analysis. It supports exploratory data analysis 
  through descriptive statistics, data visualization, statistical 
  tests (e.g., normality assessment), linear modeling, data 
  import, transformation and reporting. For more details see Shapiro and Wilk 
  (1965) <doi:10.2307/2333709>.
License: AGPL (>= 3)
URL: https://github.com/lgschuck/spada,
        https://lgschuck.github.io/spada/
BugReports: https://github.com/lgschuck/spada/issues
Depends: R (>= 4.4.0)
Imports: bsicons (>= 0.1.2), bslib (>= 0.10.0), collapse (>= 2.1.6),
        data.table (>= 1.18.2.1), DescTools (>= 0.99.60), ggplot2 (>=
        4.0.1), gt (>= 1.3.0), haven (>= 2.5.5), htmltools (>= 0.5.9),
        mirai (>= 2.5.3), qs2 (>= 0.1.7), rlang (>= 1.1.7), shiny (>=
        1.12.1), shinyWidgets (>= 0.9.0), waiter (>= 0.2.5.1), writexl
        (>= 1.5.4)
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: dplyr, knitr, rmarkdown, testthat (>= 3.0.0), tidyselect
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-06-11 10:38:57 UTC; luisg
Author: Luís Gustavo Schuck [aut, cre]
Repository: CRAN
Date/Publication: 2026-06-11 11:20:02 UTC
Built: R 4.5.2; ; 2026-06-11 15:35:21 UTC; unix
