# Generated by roxygen2: do not edit by hand

export(spada)
import(shiny)
importFrom(DescTools,Freq)
importFrom(DescTools,Gmean)
importFrom(DescTools,Hmean)
importFrom(DescTools,Kurt)
importFrom(DescTools,Mode)
importFrom(DescTools,Outlier)
importFrom(DescTools,ShapiroFranciaTest)
importFrom(DescTools,Skew)
importFrom(DescTools,StrCap)
importFrom(DescTools,VIF)
importFrom(DescTools,ZTest)
importFrom(bsicons,bs_icon)
importFrom(bslib,accordion)
importFrom(bslib,accordion_panel)
importFrom(bslib,bind_task_button)
importFrom(bslib,bs_add_rules)
importFrom(bslib,bs_theme)
importFrom(bslib,card)
importFrom(bslib,card_body)
importFrom(bslib,card_footer)
importFrom(bslib,card_header)
importFrom(bslib,font_collection)
importFrom(bslib,input_switch)
importFrom(bslib,input_task_button)
importFrom(bslib,layout_column_wrap)
importFrom(bslib,layout_columns)
importFrom(bslib,layout_sidebar)
importFrom(bslib,nav_item)
importFrom(bslib,nav_menu)
importFrom(bslib,nav_panel)
importFrom(bslib,nav_select)
importFrom(bslib,nav_spacer)
importFrom(bslib,navbar_options)
importFrom(bslib,navset_card_pill)
importFrom(bslib,page_navbar)
importFrom(bslib,popover)
importFrom(bslib,sidebar)
importFrom(bslib,tooltip)
importFrom(bslib,update_switch)
importFrom(bslib,value_box)
importFrom(collapse,"%==%")
importFrom(collapse,allNA)
importFrom(collapse,ffirst)
importFrom(collapse,flast)
importFrom(collapse,fmax)
importFrom(collapse,fmean)
importFrom(collapse,fmedian)
importFrom(collapse,fmin)
importFrom(collapse,fmode)
importFrom(collapse,fncol)
importFrom(collapse,fnobs)
importFrom(collapse,fnrow)
importFrom(collapse,fnunique)
importFrom(collapse,fquantile)
importFrom(collapse,frange)
importFrom(collapse,fsd)
importFrom(collapse,fsum)
importFrom(collapse,fvar)
importFrom(collapse,qtab)
importFrom(collapse,whichNA)
importFrom(data.table,"%between%")
importFrom(data.table,"%notin%")
importFrom(data.table,":=")
importFrom(data.table,.SD)
importFrom(data.table,as.data.table)
importFrom(data.table,between)
importFrom(data.table,copy)
importFrom(data.table,data.table)
importFrom(data.table,fcase)
importFrom(data.table,fcoalesce)
importFrom(data.table,fifelse)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(data.table,hour)
importFrom(data.table,is.data.table)
importFrom(data.table,mday)
importFrom(data.table,melt)
importFrom(data.table,minute)
importFrom(data.table,month)
importFrom(data.table,quarter)
importFrom(data.table,rbindlist)
importFrom(data.table,second)
importFrom(data.table,setDT)
importFrom(data.table,setcolorder)
importFrom(data.table,setnames)
importFrom(data.table,setorderv)
importFrom(data.table,shift)
importFrom(data.table,wday)
importFrom(data.table,week)
importFrom(data.table,yday)
importFrom(data.table,year)
importFrom(ggplot2,.data)
importFrom(ggplot2,aes)
importFrom(ggplot2,after_stat)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,stat_boxplot)
importFrom(ggplot2,stat_function)
importFrom(ggplot2,stat_qq)
importFrom(ggplot2,stat_qq_line)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,ylim)
importFrom(grDevices,colors)
importFrom(graphics,abline)
importFrom(graphics,barplot)
importFrom(graphics,boxplot)
importFrom(graphics,curve)
importFrom(graphics,hist)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,polygon)
importFrom(graphics,text)
importFrom(gt,as_raw_html)
importFrom(gt,cols_align)
importFrom(gt,cols_hide)
importFrom(gt,cols_label)
importFrom(gt,cols_merge)
importFrom(gt,cols_move)
importFrom(gt,cols_width)
importFrom(gt,data_color)
importFrom(gt,fmt_bytes)
importFrom(gt,fmt_icon)
importFrom(gt,fmt_integer)
importFrom(gt,fmt_number)
importFrom(gt,fmt_percent)
importFrom(gt,gt)
importFrom(gt,gt_output)
importFrom(gt,gtsave)
importFrom(gt,opt_interactive)
importFrom(gt,pct)
importFrom(gt,render_gt)
importFrom(gt,sub_missing)
importFrom(gt,sub_values)
importFrom(gt,tab_header)
importFrom(gt,tab_options)
importFrom(gt,tab_spanner)
importFrom(haven,as_factor)
importFrom(haven,is.labelled)
importFrom(haven,read_sav)
importFrom(haven,write_sav)
importFrom(htmltools,plotTag)
importFrom(htmltools,save_html)
importFrom(mirai,daemons)
importFrom(mirai,mirai)
importFrom(qs2,qs_read)
importFrom(qs2,qs_save)
importFrom(rlang,parse_expr)
importFrom(shinyWidgets,colorPickr)
importFrom(shinyWidgets,dropdownButton)
importFrom(shinyWidgets,radioGroupButtons)
importFrom(shinyWidgets,show_toast)
importFrom(shinyWidgets,statiCard)
importFrom(shinyWidgets,updateColorPickr)
importFrom(shinyWidgets,updateRadioGroupButtons)
importFrom(stats,IQR)
importFrom(stats,cor)
importFrom(stats,cor.test)
importFrom(stats,dnorm)
importFrom(stats,dt)
importFrom(stats,formula)
importFrom(stats,ks.test)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,pf)
importFrom(stats,qnorm)
importFrom(stats,qqline)
importFrom(stats,qqnorm)
importFrom(stats,qt)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,shapiro.test)
importFrom(stats,t.test)
importFrom(stats,terms)
importFrom(stats,var)
importFrom(tools,R_user_dir)
importFrom(tools,file_ext)
importFrom(tools,toTitleCase)
importFrom(utils,head)
importFrom(utils,object.size)
importFrom(utils,packageDescription)
importFrom(utils,packageVersion)
importFrom(utils,sessionInfo)
importFrom(waiter,useWaiter)
importFrom(waiter,waiterShowOnLoad)
importFrom(waiter,waiter_hide)
importFrom(waiter,waiter_show)
importFrom(writexl,write_xlsx)
