
utils::globalVariables(
  c('.N',
    'cols',
    'density',
    'dt_names_react',
    'dup',
    'f_icon',
    'fun',
    'groupby',
    'i',
    'j',
    'label',
    'measure',
    'Measures',
    'new_var',
    'n_nas',
    'n_unique',
    'n_valid',
    'n_zero',
    'perc_nas',
    'perc_unique',
    'perc_valid',
    'perc_zero',
    'preview',
    'Pr(>|t|)',
    'q1',
    'q3',
    'results',
    'rows',
    'sel_var',
    'Sig Levels',
    'size',
    'terms',
    'value',
    'values',
    'var1',
    'Var1',
    'Var2',
    'x',
    'y',
    'integer_var',
    'numeric_var',
    'char_var',
    'char_long_var',
    'char_colors_var',
    'date_var',
    'datetime_var',
    'factor_var',
    'logical_var',
    'posixct_var',
    'waiter_right_foot'
    )
  )
